#!/usr/bin/env -S npx tsx

// src/bin/bin.ts
import { cwd as cwd2 } from "node:process";

// src/bootstrap/load.ts
import { join } from "node:path";
import { consola as consola3 } from "consola";

// src/command/command.ts
import path4 from "node:path";
import chokidar from "chokidar";

// ../../node_modules/.pnpm/citty@0.1.6/node_modules/citty/dist/index.mjs
import consola from "consola";
import { colors } from "consola/utils";
function toArray(val) {
  if (Array.isArray(val)) {
    return val;
  }
  return val === void 0 ? [] : [val];
}
function formatLineColumns(lines, linePrefix = "") {
  const maxLengh = [];
  for (const line of lines) {
    for (const [i, element] of line.entries()) {
      maxLengh[i] = Math.max(maxLengh[i] || 0, element.length);
    }
  }
  return lines.map(
    (l) => l.map(
      (c, i) => linePrefix + c[i === 0 ? "padStart" : "padEnd"](maxLengh[i])
    ).join("  ")
  ).join("\n");
}
function resolveValue(input) {
  return typeof input === "function" ? input() : input;
}
var CLIError = class extends Error {
  constructor(message, code) {
    super(message);
    this.code = code;
    this.name = "CLIError";
  }
};
var NUMBER_CHAR_RE = /\d/;
var STR_SPLITTERS = ["-", "_", "/", "."];
function isUppercase(char = "") {
  if (NUMBER_CHAR_RE.test(char)) {
    return void 0;
  }
  return char !== char.toLowerCase();
}
function splitByCase(str, separators) {
  const splitters = separators ?? STR_SPLITTERS;
  const parts = [];
  if (!str || typeof str !== "string") {
    return parts;
  }
  let buff = "";
  let previousUpper;
  let previousSplitter;
  for (const char of str) {
    const isSplitter = splitters.includes(char);
    if (isSplitter === true) {
      parts.push(buff);
      buff = "";
      previousUpper = void 0;
      continue;
    }
    const isUpper = isUppercase(char);
    if (previousSplitter === false) {
      if (previousUpper === false && isUpper === true) {
        parts.push(buff);
        buff = char;
        previousUpper = isUpper;
        continue;
      }
      if (previousUpper === true && isUpper === false && buff.length > 1) {
        const lastChar = buff.at(-1);
        parts.push(buff.slice(0, Math.max(0, buff.length - 1)));
        buff = lastChar + char;
        previousUpper = isUpper;
        continue;
      }
    }
    buff += char;
    previousUpper = isUpper;
    previousSplitter = isSplitter;
  }
  parts.push(buff);
  return parts;
}
function upperFirst(str) {
  return str ? str[0].toUpperCase() + str.slice(1) : "";
}
function lowerFirst(str) {
  return str ? str[0].toLowerCase() + str.slice(1) : "";
}
function pascalCase(str, opts) {
  return str ? (Array.isArray(str) ? str : splitByCase(str)).map((p) => upperFirst(opts?.normalize ? p.toLowerCase() : p)).join("") : "";
}
function camelCase(str, opts) {
  return lowerFirst(pascalCase(str || "", opts));
}
function kebabCase(str, joiner) {
  return str ? (Array.isArray(str) ? str : splitByCase(str)).map((p) => p.toLowerCase()).join(joiner ?? "-") : "";
}
function toArr(any) {
  return any == void 0 ? [] : Array.isArray(any) ? any : [any];
}
function toVal(out, key, val, opts) {
  let x;
  const old = out[key];
  const nxt = ~opts.string.indexOf(key) ? val == void 0 || val === true ? "" : String(val) : typeof val === "boolean" ? val : ~opts.boolean.indexOf(key) ? val === "false" ? false : val === "true" || (out._.push((x = +val, x * 0 === 0) ? x : val), !!val) : (x = +val, x * 0 === 0) ? x : val;
  out[key] = old == void 0 ? nxt : Array.isArray(old) ? old.concat(nxt) : [old, nxt];
}
function parseRawArgs(args = [], opts = {}) {
  let k;
  let arr;
  let arg;
  let name;
  let val;
  const out = { _: [] };
  let i = 0;
  let j = 0;
  let idx = 0;
  const len = args.length;
  const alibi = opts.alias !== void 0;
  const strict = opts.unknown !== void 0;
  const defaults = opts.default !== void 0;
  opts.alias = opts.alias || {};
  opts.string = toArr(opts.string);
  opts.boolean = toArr(opts.boolean);
  if (alibi) {
    for (k in opts.alias) {
      arr = opts.alias[k] = toArr(opts.alias[k]);
      for (i = 0; i < arr.length; i++) {
        (opts.alias[arr[i]] = arr.concat(k)).splice(i, 1);
      }
    }
  }
  for (i = opts.boolean.length; i-- > 0; ) {
    arr = opts.alias[opts.boolean[i]] || [];
    for (j = arr.length; j-- > 0; ) {
      opts.boolean.push(arr[j]);
    }
  }
  for (i = opts.string.length; i-- > 0; ) {
    arr = opts.alias[opts.string[i]] || [];
    for (j = arr.length; j-- > 0; ) {
      opts.string.push(arr[j]);
    }
  }
  if (defaults) {
    for (k in opts.default) {
      name = typeof opts.default[k];
      arr = opts.alias[k] = opts.alias[k] || [];
      if (opts[name] !== void 0) {
        opts[name].push(k);
        for (i = 0; i < arr.length; i++) {
          opts[name].push(arr[i]);
        }
      }
    }
  }
  const keys = strict ? Object.keys(opts.alias) : [];
  for (i = 0; i < len; i++) {
    arg = args[i];
    if (arg === "--") {
      out._ = out._.concat(args.slice(++i));
      break;
    }
    for (j = 0; j < arg.length; j++) {
      if (arg.charCodeAt(j) !== 45) {
        break;
      }
    }
    if (j === 0) {
      out._.push(arg);
    } else if (arg.substring(j, j + 3) === "no-") {
      name = arg.slice(Math.max(0, j + 3));
      if (strict && !~keys.indexOf(name)) {
        return opts.unknown(arg);
      }
      out[name] = false;
    } else {
      for (idx = j + 1; idx < arg.length; idx++) {
        if (arg.charCodeAt(idx) === 61) {
          break;
        }
      }
      name = arg.substring(j, idx);
      val = arg.slice(Math.max(0, ++idx)) || i + 1 === len || ("" + args[i + 1]).charCodeAt(0) === 45 || args[++i];
      arr = j === 2 ? [name] : name;
      for (idx = 0; idx < arr.length; idx++) {
        name = arr[idx];
        if (strict && !~keys.indexOf(name)) {
          return opts.unknown("-".repeat(j) + name);
        }
        toVal(out, name, idx + 1 < arr.length || val, opts);
      }
    }
  }
  if (defaults) {
    for (k in opts.default) {
      if (out[k] === void 0) {
        out[k] = opts.default[k];
      }
    }
  }
  if (alibi) {
    for (k in out) {
      arr = opts.alias[k] || [];
      while (arr.length > 0) {
        out[arr.shift()] = out[k];
      }
    }
  }
  return out;
}
function parseArgs(rawArgs, argsDef) {
  const parseOptions = {
    boolean: [],
    string: [],
    mixed: [],
    alias: {},
    default: {}
  };
  const args = resolveArgs(argsDef);
  for (const arg of args) {
    if (arg.type === "positional") {
      continue;
    }
    if (arg.type === "string") {
      parseOptions.string.push(arg.name);
    } else if (arg.type === "boolean") {
      parseOptions.boolean.push(arg.name);
    }
    if (arg.default !== void 0) {
      parseOptions.default[arg.name] = arg.default;
    }
    if (arg.alias) {
      parseOptions.alias[arg.name] = arg.alias;
    }
  }
  const parsed = parseRawArgs(rawArgs, parseOptions);
  const [...positionalArguments] = parsed._;
  const parsedArgsProxy = new Proxy(parsed, {
    get(target, prop) {
      return target[prop] ?? target[camelCase(prop)] ?? target[kebabCase(prop)];
    }
  });
  for (const [, arg] of args.entries()) {
    if (arg.type === "positional") {
      const nextPositionalArgument = positionalArguments.shift();
      if (nextPositionalArgument !== void 0) {
        parsedArgsProxy[arg.name] = nextPositionalArgument;
      } else if (arg.default === void 0 && arg.required !== false) {
        throw new CLIError(
          `Missing required positional argument: ${arg.name.toUpperCase()}`,
          "EARG"
        );
      } else {
        parsedArgsProxy[arg.name] = arg.default;
      }
    } else if (arg.required && parsedArgsProxy[arg.name] === void 0) {
      throw new CLIError(`Missing required argument: --${arg.name}`, "EARG");
    }
  }
  return parsedArgsProxy;
}
function resolveArgs(argsDef) {
  const args = [];
  for (const [name, argDef] of Object.entries(argsDef || {})) {
    args.push({
      ...argDef,
      name,
      alias: toArray(argDef.alias)
    });
  }
  return args;
}
function defineCommand(def) {
  return def;
}
async function runCommand(cmd, opts) {
  const cmdArgs = await resolveValue(cmd.args || {});
  const parsedArgs = parseArgs(opts.rawArgs, cmdArgs);
  const context = {
    rawArgs: opts.rawArgs,
    args: parsedArgs,
    data: opts.data,
    cmd
  };
  if (typeof cmd.setup === "function") {
    await cmd.setup(context);
  }
  let result;
  try {
    const subCommands = await resolveValue(cmd.subCommands);
    if (subCommands && Object.keys(subCommands).length > 0) {
      const subCommandArgIndex = opts.rawArgs.findIndex(
        (arg) => !arg.startsWith("-")
      );
      const subCommandName = opts.rawArgs[subCommandArgIndex];
      if (subCommandName) {
        if (!subCommands[subCommandName]) {
          throw new CLIError(
            `Unknown command \`${subCommandName}\``,
            "E_UNKNOWN_COMMAND"
          );
        }
        const subCommand = await resolveValue(subCommands[subCommandName]);
        if (subCommand) {
          await runCommand(subCommand, {
            rawArgs: opts.rawArgs.slice(subCommandArgIndex + 1)
          });
        }
      } else if (!cmd.run) {
        throw new CLIError(`No command specified.`, "E_NO_COMMAND");
      }
    }
    if (typeof cmd.run === "function") {
      result = await cmd.run(context);
    }
  } finally {
    if (typeof cmd.cleanup === "function") {
      await cmd.cleanup(context);
    }
  }
  return { result };
}
async function resolveSubCommand(cmd, rawArgs, parent) {
  const subCommands = await resolveValue(cmd.subCommands);
  if (subCommands && Object.keys(subCommands).length > 0) {
    const subCommandArgIndex = rawArgs.findIndex((arg) => !arg.startsWith("-"));
    const subCommandName = rawArgs[subCommandArgIndex];
    const subCommand = await resolveValue(subCommands[subCommandName]);
    if (subCommand) {
      return resolveSubCommand(
        subCommand,
        rawArgs.slice(subCommandArgIndex + 1),
        cmd
      );
    }
  }
  return [cmd, parent];
}
async function showUsage(cmd, parent) {
  try {
    consola.log(await renderUsage(cmd, parent) + "\n");
  } catch (error) {
    consola.error(error);
  }
}
async function renderUsage(cmd, parent) {
  const cmdMeta = await resolveValue(cmd.meta || {});
  const cmdArgs = resolveArgs(await resolveValue(cmd.args || {}));
  const parentMeta = await resolveValue(parent?.meta || {});
  const commandName = `${parentMeta.name ? `${parentMeta.name} ` : ""}` + (cmdMeta.name || process.argv[1]);
  const argLines = [];
  const posLines = [];
  const commandsLines = [];
  const usageLine = [];
  for (const arg of cmdArgs) {
    if (arg.type === "positional") {
      const name = arg.name.toUpperCase();
      const isRequired = arg.required !== false && arg.default === void 0;
      const defaultHint = arg.default ? `="${arg.default}"` : "";
      posLines.push([
        "`" + name + defaultHint + "`",
        arg.description || "",
        arg.valueHint ? `<${arg.valueHint}>` : ""
      ]);
      usageLine.push(isRequired ? `<${name}>` : `[${name}]`);
    } else {
      const isRequired = arg.required === true && arg.default === void 0;
      const argStr = (arg.type === "boolean" && arg.default === true ? [
        ...(arg.alias || []).map((a) => `--no-${a}`),
        `--no-${arg.name}`
      ].join(", ") : [...(arg.alias || []).map((a) => `-${a}`), `--${arg.name}`].join(
        ", "
      )) + (arg.type === "string" && (arg.valueHint || arg.default) ? `=${arg.valueHint ? `<${arg.valueHint}>` : `"${arg.default || ""}"`}` : "");
      argLines.push([
        "`" + argStr + (isRequired ? " (required)" : "") + "`",
        arg.description || ""
      ]);
      if (isRequired) {
        usageLine.push(argStr);
      }
    }
  }
  if (cmd.subCommands) {
    const commandNames = [];
    const subCommands = await resolveValue(cmd.subCommands);
    for (const [name, sub] of Object.entries(subCommands)) {
      const subCmd = await resolveValue(sub);
      const meta = await resolveValue(subCmd?.meta);
      commandsLines.push([`\`${name}\``, meta?.description || ""]);
      commandNames.push(name);
    }
    usageLine.push(commandNames.join("|"));
  }
  const usageLines = [];
  const version = cmdMeta.version || parentMeta.version;
  usageLines.push(
    colors.gray(
      `${cmdMeta.description} (${commandName + (version ? ` v${version}` : "")})`
    ),
    ""
  );
  const hasOptions = argLines.length > 0 || posLines.length > 0;
  usageLines.push(
    `${colors.underline(colors.bold("USAGE"))} \`${commandName}${hasOptions ? " [OPTIONS]" : ""} ${usageLine.join(" ")}\``,
    ""
  );
  if (posLines.length > 0) {
    usageLines.push(colors.underline(colors.bold("ARGUMENTS")), "");
    usageLines.push(formatLineColumns(posLines, "  "));
    usageLines.push("");
  }
  if (argLines.length > 0) {
    usageLines.push(colors.underline(colors.bold("OPTIONS")), "");
    usageLines.push(formatLineColumns(argLines, "  "));
    usageLines.push("");
  }
  if (commandsLines.length > 0) {
    usageLines.push(colors.underline(colors.bold("COMMANDS")), "");
    usageLines.push(formatLineColumns(commandsLines, "  "));
    usageLines.push(
      "",
      `Use \`${commandName} <command> --help\` for more information about a command.`
    );
  }
  return usageLines.filter((l) => typeof l === "string").join("\n");
}
async function runMain(cmd, opts = {}) {
  const rawArgs = opts.rawArgs || process.argv.slice(2);
  const showUsage$1 = opts.showUsage || showUsage;
  try {
    if (rawArgs.includes("--help") || rawArgs.includes("-h")) {
      await showUsage$1(...await resolveSubCommand(cmd, rawArgs));
      process.exit(0);
    } else if (rawArgs.length === 1 && rawArgs[0] === "--version") {
      const meta = typeof cmd.meta === "function" ? await cmd.meta() : await cmd.meta;
      if (!meta?.version) {
        throw new CLIError("No version specified", "E_NO_VERSION");
      }
      consola.log(meta.version);
    } else {
      await runCommand(cmd, { rawArgs });
    }
  } catch (error) {
    const isCLIError = error instanceof CLIError;
    if (!isCLIError) {
      consola.error(error, "\n");
    }
    if (isCLIError) {
      await showUsage$1(...await resolveSubCommand(cmd, rawArgs));
    }
    consola.error(error.message);
    process.exit(1);
  }
}

// src/command/command.ts
import { $ as $2 } from "execa";
import fs4 from "fs-extra";

// src/bootstrap/config.ts
import path2 from "node:path";
import { cwd } from "node:process";
import defu from "defu";

// src/log.ts
import consola2, { LogLevels } from "consola";
function isLogger(m) {
  return typeof m === "object" && m !== null && "log" in m;
}
function getConsolaLevel(level) {
  switch (level) {
    case "error":
      return 0;
    case "warn":
      return 1;
    case "normal":
      return 2;
    case "info":
      return 3;
    case "debug":
      return 4;
    case "trace":
      return 5;
    default:
      return 3;
  }
}
function getLogger(name) {
  let config2;
  try {
    config2 = getConfig();
  } catch (e) {
  }
  consola2.level = getConsolaLevel(
    config2?.logger?.level ?? defaultConfig.logger.level
  );
  if (process.env.DEBUG) {
    consola2.level = LogLevels.debug;
  }
  return name ? consola2.withTag(name) : consola2;
}

// src/bootstrap/file-module.ts
import fs2 from "node:fs/promises";
import { readdir } from "node:fs/promises";
import path from "node:path";
import { pathToFileURL } from "node:url";

// src/plainstack-fs.ts
import fs from "node:fs/promises";
async function fileExists(filePath) {
  try {
    await fs.access(filePath);
    return true;
  } catch {
    return false;
  }
}
async function directoryExists(dirPath) {
  try {
    await fs.access(dirPath);
    return true;
  } catch {
    return false;
  }
}
async function ensureDirectoryExists(dirPath) {
  try {
    await fs.access(dirPath);
  } catch {
    await fs.mkdir(dirPath, { recursive: true });
  }
}

// src/bootstrap/file-module.ts
async function loadModule(filePath, load2) {
  const log = getLogger("load-module");
  if (!await fileExists(filePath)) {
    log.debug(`tried loading module, ${filePath} does not exist`);
    return void 0;
  }
  const module = await import(pathToFileURL(filePath).toString());
  const result = {
    namedExports: {}
  };
  if ("default" in module) {
    const defaultExport = module.default;
    if (defaultExport && typeof defaultExport === "object" && "default" in defaultExport) {
      result.defaultExport = await load2(defaultExport.default);
    } else {
      result.defaultExport = await load2(defaultExport);
    }
  }
  for (const [key, value] of Object.entries(module)) {
    if (key !== "default") {
      result.namedExports[key] = await load2(value);
    }
  }
  if (result.defaultExport)
    log.debug(`successfully loaded default export in module ${filePath}`);
  if (Object.keys(result.namedExports).length)
    log.debug(`successfully loaded named exports in module ${filePath}`);
  return result;
}
async function loadModulesFromDir(baseDir, load2, extensions = [".ts", ".tsx"], currentDir = baseDir) {
  const modules = [];
  const log = getLogger("manifest");
  if (!await directoryExists(currentDir)) {
    log.debug(`tried loading modules, directory ${currentDir} does not exist`);
    return [];
  }
  const files = await readdir(currentDir);
  log.debug(`found ${files.length} files in ${currentDir}`);
  for (const file of files) {
    const absolutePath = path.join(currentDir, file);
    const stat = await fs2.stat(absolutePath);
    if (stat.isDirectory()) {
      log.debug(`found directory: ${absolutePath}`);
      const subModules = await loadModulesFromDir(
        baseDir,
        load2,
        extensions,
        absolutePath
      );
      modules.push(...subModules);
    } else if (stat.isFile()) {
      log.debug(`found file: ${absolutePath}`);
      if (!extensions.includes(path.extname(file))) {
        log.debug(`skipping file ${absolutePath} with unsupported extension`);
        continue;
      }
      const relativePath = path.relative(baseDir, absolutePath);
      const result = await loadModule(absolutePath, load2);
      if (!result) continue;
      modules.push({
        defaultExport: result.defaultExport,
        namedExports: result.namedExports,
        filename: path.parse(file).name,
        extension: path.extname(file),
        absolutePath,
        relativePath
      });
    }
  }
  return modules;
}

// src/bootstrap/config.ts
var config;
function isConfig(m) {
  return typeof m === "object" && m !== null && "nodeEnv" in m;
}
async function load(m) {
  if (!isConfig(m)) throw new Error("invalid config module");
  return m;
}
var defaultConfig = {
  nodeEnv: "production",
  logger: {
    level: "normal"
  },
  http: {
    port: 3e3,
    staticPath: "/public"
  },
  paths: {
    routes: "app/routes",
    commands: "app/commands",
    jobs: "app/jobs",
    schedules: "app/schedules",
    schema: "app/config/schema.ts",
    styles: "assets/styles.css",
    assets: "assets",
    out: ".out",
    seed: "database/seed.ts",
    migrations: "database/migrations",
    database: "app/config/database.ts",
    http: "app/config/http.ts",
    queue: "app/config/queue.ts",
    mailer: "app/config/mailer.ts",
    logger: "app/config/logger.ts"
  }
};
function withDefaultConfig(config2) {
  return defu(config2, defaultConfig);
}
async function loadConfig() {
  const log = getLogger("load-config");
  const fileModule = await loadModule(
    path2.join(cwd(), "plainstack.config.ts"),
    load
  );
  if (!fileModule) throw new Error("no plainstack.config.ts found");
  if (!fileModule.defaultExport)
    throw new Error("no default export found in plainstack.config.ts");
  config = withDefaultConfig(fileModule.defaultExport);
  log.debug("config loaded from file", config);
}
function getConfig() {
  if (!config)
    throw new Error("config not loaded, make sure to call loadConfig() first");
  return config;
}
async function loadAndGetConfig() {
  const log = getLogger("load-config");
  if (config) {
    log.debug("return memoized config");
    return config;
  }
  await loadConfig();
  return getConfig();
}

// src/bootstrap/get.ts
var appLoader;
function setAppLoader(loader) {
  appLoader = loader;
}
async function get(keys) {
  const log = getLogger("app-loader");
  if (!appLoader)
    throw new Error(
      "appLoader not set, make sure to call registerAppLoader() first"
    );
  log.debug(`getting app components for ${keys.join(", ")}`);
  const results = await Promise.all(
    keys.map(async (key) => {
      try {
        const loader = appLoader?.[key];
        if (!loader) {
          log.warn(`no loader found for ${key}`);
          return [key, void 0];
        }
        log.debug(`loading app component: ${key}`);
        const result = await loader();
        log.debug(`successfully loaded app component: ${key}`);
        return [key, result];
      } catch (error) {
        log.error(`failed to load app component: ${key}`, error);
        return [key, void 0];
      }
    })
  );
  return Object.fromEntries(results);
}
async function getOrThrow(keys) {
  const log = getLogger("app-loader");
  if (!appLoader) {
    throw new Error("appLoader not set. call setAppLoader first.");
  }
  log.debug(`getting app components (or throw) for ${keys.join(", ")}`);
  const result = await get(keys);
  const missingKeys = keys.filter((key) => result[key] === void 0);
  if (missingKeys.length > 0) {
    throw new Error(`missing app components: ${missingKeys.join(", ")}`);
  }
  return result;
}

// src/database/database.ts
import fs3 from "node:fs/promises";
import path3 from "node:path";
import { Migrator, sql } from "kysely";
import { TSFileMigrationProvider } from "kysely-ctl";
async function getMigrator() {
  const config2 = await loadAndGetConfig();
  const { database } = await getOrThrow(["database"]);
  return new Migrator({
    db: database,
    provider: new TSFileMigrationProvider({
      migrationFolder: path3.join(process.cwd(), config2.paths.migrations)
    })
  });
}
async function hasPendingMigrations() {
  const migrator = await getMigrator();
  const migrations = await migrator.getMigrations();
  return migrations.some((m) => !m.executedAt);
}
async function migrateToLatest() {
  const log = getLogger("database");
  const migrator = await getMigrator();
  log.info("running migrations");
  const result = await migrator.migrateToLatest();
  log.info(`applied ${result?.results?.length} migrations`);
  if (result?.error) log.error("migration error:", result.error);
}
var migrationFileTemplate = `
import type { Kysely } from "kysely";

// check https://kysely.dev/docs/migrations
export async function up(db: Kysely<unknown>): Promise<void> {
  await db.schema
    .createTable("person")
    .addColumn("id", "varchar", (col) => col.primaryKey())
    .addColumn("first_name", "text", (col) => col.notNull())
    .addColumn("last_name", "text")
    .addColumn("gender", "text", (col) => col.notNull())
    .addColumn("created_at", "integer", (col) => col.notNull())
    .execute();

  await db.schema
    .createTable("pet")
    .addColumn("id", "varchar", (col) => col.primaryKey())
    .addColumn("name", "text", (col) => col.notNull().unique())
    .addColumn("owner_id", "varchar", (col) => col.notNull())
    .addColumn("species", "text", (col) => col.notNull())
    .addColumn("created_at", "integer", (col) => col.notNull())
    .execute();

  await db.schema
    .createIndex("pet_owner_id_index")
    .on("pet")
    .column("owner_id")
    .execute();
}

export async function down(db: Kysely<unknown>): Promise<void> {
  await db.schema.dropTable("pet").execute();
  await db.schema.dropTable("person").execute();
}
`;
async function writeMigrationFile(name) {
  const log = getLogger("database");
  const config2 = await loadAndGetConfig();
  const sanitizedName = name.toLowerCase().replace(/[^a-z0-9_]/g, "_");
  const timestamp = Date.now();
  const fileName = `${timestamp}_${sanitizedName}.ts`;
  await ensureDirectoryExists(config2.paths.migrations);
  const filePath = path3.join(config2.paths.migrations, fileName);
  if (await fileExists(filePath)) {
    throw new Error(`Migration file ${fileName} already exists`);
  }
  await fs3.writeFile(filePath, migrationFileTemplate);
  log.info(`Generated migration: ${fileName}`);
}
function isDatabase(db) {
  return typeof db === "object" && db !== null && "schema" in db && "selectFrom" in db;
}

// src/database/seed.ts
function isSeeder(m) {
  return typeof m === "object" && m !== null && "run" in m;
}
async function runSeed() {
  const now = Date.now();
  const log = getLogger("seed");
  const { database, seeder } = await getOrThrow(["database", "seeder"]);
  await seeder.run(database);
  log.log(`seeds ran in ${Date.now() - now}ms`);
}

// src/web/http.ts
import expressListEndpoints from "express-list-endpoints";
import expressRateLimit from "express-rate-limit";
import morgan from "morgan";

// src/id.ts
import { createId } from "@paralleldrive/cuid2";

// src/web/file-router.ts
import express from "express";

// src/web/plain-response.ts
import "@kitajs/html";
import { renderToStream } from "@kitajs/html/suspense";

// src/web/handler.ts
function isHandler(m) {
  return typeof m === "function";
}

// src/web/file-router.ts
function getFileRoutesFromFileModules(modules) {
  const log = getLogger("file-router");
  const fileRoutes = [];
  for (const module of modules) {
    const fileRoute = {
      filePath: module.relativePath,
      GET: void 0,
      POST: void 0
    };
    if (!module.namedExports?.GET && !module.namedExports?.POST && !module.defaultExport) {
      throw new Error(
        `invalid file route module at ${module.absolutePath}, no GET or POST export found`
      );
    }
    if (module.namedExports?.GET) {
      log.debug(`found GET export in ${module.absolutePath}`);
      fileRoute.GET = module.namedExports.GET;
    }
    if (module.namedExports?.POST) {
      log.debug(`found POST export in ${module.absolutePath}`);
      fileRoute.POST = module.namedExports.POST;
    }
    if (module.defaultExport && !fileRoute.GET) {
      log.debug(
        `found default export in ${module.absolutePath}, assuming GET export`
      );
      fileRoute.GET = module.defaultExport;
    }
    fileRoutes.push(fileRoute);
  }
  return fileRoutes;
}

// src/web/http.ts
async function printRoutes(app) {
  const endpoints = expressListEndpoints(app);
  const sorted = endpoints.sort((a, b) => {
    if (a.path < b.path) return -1;
    if (a.path > b.path) return 1;
    return 0;
  });
  console.log("Routes:");
  for (const endpoint of sorted) {
    for (const method of endpoint.methods) {
      console.log(method, endpoint.path);
    }
  }
}
function isHttp(m) {
  return typeof m === "function";
}

// src/command/info.ts
import { $ } from "execa";

// package.json
var package_default = {
  name: "plainstack",
  version: "0.0.19",
  description: "A framework combining HTMX, SQLite and TypeScript for less complexity and more joy",
  type: "module",
  main: "dist/plainstack.js",
  types: "dist/plainstack.d.ts",
  bin: {
    plainstack: "dist/bin/bin.js",
    "plainstack-dev": "dist/bin/bin-dev.js",
    "plainstack-work": "dist/bin/bin-work.js"
  },
  scripts: {
    test: "vitest run",
    coverage: "vitest run --coverage",
    build: "tsc --noEmit && tsup"
  },
  keywords: [
    "plainweb",
    "plainstack",
    "sqlite",
    "typescript",
    "htmx",
    "express",
    "typescript"
  ],
  author: "Josef Erben <josef@plainweb.dev>",
  license: "MIT",
  peerDependencies: {
    "@kitajs/html": "^4.2.1",
    esbuild: "^0.23.1",
    express: "^4.19.2",
    kysely: "^0.27.4",
    "kysely-codegen": "^0.16.5",
    tailwindcss: "^3.4.3",
    tsx: "^4.16.2",
    typescript: "^5.3.3",
    vitest: "^2.0.1",
    zod: "^3.23.8"
  },
  dependencies: {
    "@paralleldrive/cuid2": "^2.2.2",
    chokidar: "^4.0.0",
    consola: "^3.2.3",
    defu: "^6.1.4",
    dotenv: "^16.4.5",
    execa: "^9.3.1",
    "express-list-endpoints": "^7.1.0",
    "express-rate-limit": "^7.3.1",
    "fs-extra": "^11.2.0",
    "kysely-ctl": "^0.9.0",
    morgan: "^1.10.0",
    "node-mocks-http": "^1.14.1",
    nodemailer: "^6.9.13",
    plainjobs: "0.0.4"
  },
  devDependencies: {
    "@kitajs/html": "^4.2.1",
    "@kitajs/ts-html-plugin": "^4.0.1",
    "@tsconfig/node20": "^20.1.4",
    "@types/better-sqlite3": "^7.6.10",
    "@types/chokidar": "^2.1.3",
    "@types/express": "^4.17.21",
    "@types/fs-extra": "^11.0.4",
    "@types/morgan": "^1.9.9",
    "@types/node": "^20",
    "@types/nodemailer": "^6.4.15",
    "@types/supertest": "^6.0.2",
    "@vitest/coverage-v8": "^2.0.3",
    "better-sqlite3": "^11.0.0",
    citty: "^0.1.6",
    esbuild: "^0.23.1",
    express: "^4.19.2",
    kysely: "^0.27.4",
    "kysely-codegen": "^0.16.5",
    supertest: "^7.0.0",
    tailwindcss: "^3.4.10",
    tsup: "^8.2.4",
    tsx: "^4.16.2",
    typescript: "^5.3.3",
    vitest: "^2.0.1",
    zod: "^3.23.8"
  }
};

// src/command/info.ts
async function checkBinary(binary) {
  try {
    await $`${binary} --help`;
    return true;
  } catch {
    return false;
  }
}
async function printInfo() {
  const log = getLogger("info");
  const config2 = await loadAndGetConfig();
  const app = await get([
    "database",
    "server",
    "queue",
    "jobs",
    "commands",
    "mailer",
    "schedules",
    "seeder",
    "routes"
  ]);
  log.log(`plainstack v${package_default.version}`);
  log.log("");
  log.log("app:");
  log.log(app.database ? "\u2713 database" : "\u2717 database");
  log.log(app.server ? "\u2713 http" : "\u2717 http");
  log.log(app.queue ? "\u2713 queue" : "\u2717 queue");
  log.log(app.mailer ? "\u2713 mailer" : "\u2717 mailer");
  log.log(app.seeder ? "\u2713 seeder" : "\u2717 seeder");
  if (app.jobs && Object.keys(app.jobs).length > 0) {
    log.log("\u2713 jobs:");
    for (const job of Object.keys(app.jobs)) {
      log.log(`  * ${job}`);
    }
  } else {
    log.log("\u2717 jobs");
  }
  if (app.commands && Object.keys(app.commands).length > 0) {
    log.log("\u2713 commands:");
    for (const command of Object.keys(app.commands)) {
      log.log(`  * ${command}`);
    }
  } else {
    log.log("\u2717 commands");
  }
  if (app.schedules && Object.keys(app.schedules).length > 0) {
    log.log("\u2713 schedules:");
    for (const schedule of Object.keys(app.schedules)) {
      log.log(`  * ${schedule}`);
    }
  } else {
    log.log("\u2717 schedules");
  }
  log.log(app.routes && app.routes.length > 0 ? "\u2713 routes" : "\u2717 routes");
  log.log("");
  log.log("binaries:");
  const binaries = ["esbuild", "tailwindcss", "tsx", "tsc", "biome"];
  for (const binary of binaries) {
    const installed = await checkBinary(binary);
    log.log(installed ? `\u2713 ${binary} installed` : `\u2717 ${binary} not installed`);
  }
  log.log("");
  log.log("config:");
  log.log(`  * node environment: ${config2.nodeEnv}`);
  log.log("  * logger:");
  log.log(`    - level: ${config2.logger.level}`);
  log.log("  * http:");
  log.log(`    - port: ${config2.http.port}`);
  log.log(`    - static path: ${config2.http.staticPath}`);
  log.log("  * paths:");
  for (const [key, value] of Object.entries(config2.paths)) {
    log.log(`    - ${key}: ${value}`);
  }
}

// src/command/command.ts
function isCopyableAsset(config2, src) {
  if (src.endsWith(config2.paths.styles)) return false;
  if (src.endsWith(".ts") && !src.includes(path4.sep, config2.paths.assets.length + 1))
    return false;
  return true;
}
async function copyAssets(config2) {
  const log = getLogger("copy-assets");
  return fs4.copy(config2.paths.assets, config2.paths.out, {
    filter: (src, dest) => {
      const shouldCopy = isCopyableAsset(config2, src);
      log.debug(`copy: ${src} -> ${dest}`, { shouldCopy });
      return shouldCopy;
    }
  }).catch((err) => {
    log.error("error copying assets:", err);
    throw err;
  });
}
function getBuiltInCommands({
  cwd: cwd3
  // biome-ignore lint/suspicious/noExplicitAny: <explanation>
}) {
  const log = getLogger("command");
  const build = defineCommand({
    meta: {
      name: "build",
      description: "Type-check, lint and bundle assets"
    },
    run: async () => {
      log.info("build start");
      const config2 = await loadAndGetConfig();
      const now = Date.now();
      await Promise.all([
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`biome check --fix .`.then(() => log.info("\u2713 lint")),
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`tsc --noEmit`.then(() => log.info("\u2713 types")),
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`tailwindcss -i ${config2.paths.styles} -o ${config2.paths.out}/styles.css --content "./app/**/*.{tsx,html,ts} --minify"`.then(
          () => log.info("\u2713 styles.css")
        ),
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`esbuild ${config2.paths.assets}/*.ts --bundle --outdir=${config2.paths.out} --minify --sourcemap`.then(
          () => log.info("\u2713 .ts -> .js")
        ),
        await copyAssets(config2).then(() => log.info("\u2713 assets"))
      ]);
      log.info("build took", Date.now() - now, "ms");
    }
  });
  const test2 = defineCommand({
    meta: {
      name: "test",
      description: "Run tests"
    },
    args: {
      watch: {
        type: "boolean",
        description: "Run tests in watch mode",
        default: false
      }
    },
    run: async ({ args }) => {
      if (await hasPendingMigrations()) {
        log.warn(
          "there are pending migrations, run `plainstack migrate` to apply them"
        );
      }
      await $2({
        all: true,
        preferLocal: true,
        stdout: "inherit",
        stderr: "inherit"
      })`vitest ${args.watch ? "watch" : "run"}`;
    }
  });
  const dev = defineCommand({
    meta: {
      name: "dev",
      description: "Start the local development server"
    },
    run: async () => {
      const config2 = await loadAndGetConfig();
      if (await hasPendingMigrations()) {
        log.warn(
          "there are pending migrations, run `plainstack migrate` to apply them"
        );
      }
      await Promise.all([
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`plainstack-dev`,
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`tailwindcss -i ${config2.paths.styles} -o ${config2.paths.out}/styles.css --watch --content "./app/**/*.{tsx,html,ts}"`,
        $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`esbuild ${config2.paths.assets}/*.ts --bundle --outdir=${config2.paths.out} --sourcemap`,
        copyAssets(config2).then(
          () => chokidar.watch(config2.paths.assets, {
            persistent: true,
            ignored: (path5) => {
              const shouldIgnore = !isCopyableAsset(config2, path5);
              log.debug(`watch: ${path5}`, { shouldIgnore });
              return shouldIgnore;
            }
          }).on("add", (srcPath) => {
            const relPath = path4.relative(config2.paths.assets, srcPath);
            const destPath = path4.join(config2.paths.out, relPath);
            fs4.copy(srcPath, destPath).then(
              () => log.debug(`file ${relPath} has been added and copied`)
            ).catch(
              (err) => log.error(`error copying added file ${relPath}:`, err)
            );
          }).on("change", (srcPath) => {
            const relPath = path4.relative(config2.paths.assets, srcPath);
            const destPath = path4.join(config2.paths.out, relPath);
            fs4.copy(srcPath, destPath).then(
              () => log.debug(`file ${relPath} has been changed and copied`)
            ).catch(
              (err) => log.error(`error copying changed file ${relPath}:`, err)
            );
          }).on("unlink", (srcPath) => {
            const relPath = path4.relative(config2.paths.assets, srcPath);
            const destPath = path4.join(config2.paths.out, relPath);
            fs4.remove(destPath).then(() => log.debug(`file ${relPath} has been removed`)).catch(
              (err) => log.error(`error removing file ${relPath}:`, err)
            );
          }).on("error", (error) => {
            log.error("error watching assets:", error);
          })
        )
      ]);
    }
  });
  const serve = defineCommand({
    meta: {
      name: "serve",
      description: "Start the production web server"
    },
    run: async () => {
      const config2 = await loadAndGetConfig();
      const { server } = await getOrThrow(["server"]);
      server.start();
      log.info(`\u26A1\uFE0F serving from port ${config2.http.port}`);
    }
  });
  const work = defineCommand({
    meta: {
      name: "work",
      description: "Start background workers"
    },
    args: {
      parallel: {
        type: "string",
        description: "Number of parallel workers to run",
        default: "1"
      }
    },
    run: async ({ args }) => {
      const parallel = Number.parseInt(args.parallel);
      await Promise.all(
        Array.from(
          { length: parallel },
          () => $2({
            all: true,
            preferLocal: true,
            stdout: "inherit",
            stderr: "inherit"
          })`plainstack-work`
        )
      );
    }
  });
  const routes = defineCommand({
    meta: {
      name: "routes",
      description: "Print all file routes"
    },
    run: async () => {
      const config2 = await loadAndGetConfig();
      const { server } = await getOrThrow(["server"]);
      await printRoutes(await server.start());
    }
  });
  const migrate = defineCommand({
    meta: {
      name: "migrate",
      description: "Apply all pending migrations"
    },
    args: {
      name: {
        type: "positional",
        description: "If provided, create migration with the given name",
        required: false
      },
      schema: {
        type: "boolean",
        description: "If provided, generate a new schema file based on the current database",
        required: false
      }
    },
    run: async ({ args }) => {
      const log2 = getLogger("migrate");
      if (args.schema) {
        const config2 = await loadAndGetConfig();
        if (process.env.DB_URL) process.env.DATABASE_URL = process.env.DB_URL;
        log2.info(
          `generate schema file to ${config2.paths.schema} with DATABASE_URL=${process.env.DATABASE_URL}`
        );
        await $2({
          all: true,
          preferLocal: true,
          stdout: "inherit",
          stderr: "inherit"
        })`kysely-codegen --camel-case --dialect sqlite --out-file ${config2.paths.schema}`;
      } else {
        if (args.name) {
          await writeMigrationFile(args.name);
        } else {
          await migrateToLatest();
          const config2 = await loadAndGetConfig();
          if (process.env.DB_URL) process.env.DATABASE_URL = process.env.DB_URL;
          log2.info(
            `generate schema file to ${config2.paths.schema} with DATABASE_URL=${process.env.DATABASE_URL}`
          );
          await $2({
            all: true,
            preferLocal: true,
            stdout: "inherit",
            stderr: "inherit"
          })`kysely-codegen --camel-case --dialect sqlite --out-file ${config2.paths.schema}`;
        }
      }
    }
  });
  const seed = defineCommand({
    meta: {
      name: "seed",
      description: "Run the seeds in seed.ts"
    },
    run: async () => {
      await runSeed();
    }
  });
  const info = defineCommand({
    meta: {
      name: "info",
      description: "Print project info"
    },
    run: async () => {
      await printInfo();
    }
  });
  return {
    dev,
    build,
    test: test2,
    serve,
    work,
    routes,
    migrate,
    seed,
    info
  };
}
async function getUserSubCommands({ cwd: cwd3 }) {
  const { commands } = await getOrThrow(["commands"]);
  return commands;
}
function getRootCommand({
  builtInCommands,
  cwd: cwd3
}) {
  return defineCommand({
    meta: {
      name: "plainstack",
      description: "The fastest way to build TypeScript apps"
    },
    subCommands: {
      ...builtInCommands,
      custom: {
        meta: {
          name: "custom",
          description: "Run a custom command"
        },
        subCommands: () => getUserSubCommands({ cwd: cwd3 })
      }
    }
  });
}
async function runCommand2(opts) {
  const cwd3 = opts?.cwd ?? process.cwd();
  const builtInCommands = getBuiltInCommands({ cwd: cwd3 });
  const rootCommand = getRootCommand({
    cwd: cwd3,
    builtInCommands
  });
  await runMain(rootCommand);
}
function isCommand(cmd) {
  return typeof cmd === "object" && cmd !== null && "run" in cmd;
}

// src/env.ts
import dotenv from "dotenv";
import z from "zod";
function test() {
  return process.env.NODE_ENV === "test";
}

// src/job.ts
import {
  defineQueue as definePlainjobsQueue,
  defineWorker
} from "plainjobs";
function isJob(job) {
  return typeof job === "object" && job !== null && "run" in job && "name" in job && typeof job.run === "function";
}
function isSchedule(schedule) {
  return typeof schedule === "object" && schedule !== null && "run" in schedule && "name" in schedule && typeof schedule.run === "function";
}
function isQueue(q) {
  return typeof q === "object" && q !== null && "add" in q && "schedule" in q;
}

// src/mail.ts
import nodemailer from "nodemailer";
var devMailer = nodemailer.createTransport({
  jsonTransport: true
});
function isMailer(m) {
  return typeof m === "object" && m !== null && "sendMail" in m;
}

// src/bootstrap/load.ts
function getDatabaseLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.database);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isDatabase(m))
        throw new Error(
          `default export in ${modulePath} is not a valid database`
        );
      return m;
    });
    if (!fileModule)
      throw new Error(`no default export found in ${modulePath}`);
    if (!fileModule.defaultExport)
      throw new Error(`no default export found in ${modulePath}`);
    return fileModule.defaultExport;
  };
}
function getServerLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.http);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isHttp(m))
        throw new Error(
          `default export in ${modulePath} is not a valid http server`
        );
      return m;
    });
    if (!fileModule)
      throw new Error(`no default export found in ${modulePath}`);
    if (!fileModule.defaultExport)
      throw new Error(`no default export found in ${modulePath}`);
    return {
      start: async () => {
        const app = await fileModule.defaultExport?.(config2);
        return app;
      }
    };
  };
}
function getQueueLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.queue);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isQueue(m))
        throw new Error(`default export in ${modulePath} is not a valid queue`);
      return m;
    });
    if (!fileModule)
      throw new Error(`no default export found in ${modulePath}`);
    if (!fileModule.defaultExport)
      throw new Error(`no default export found in ${modulePath}`);
    const namedExports = {};
    for (const [key, value] of Object.entries(fileModule.namedExports)) {
      if (!isQueue(value))
        throw new Error(
          `named export ${key} in ${modulePath} is not a valid queue`
        );
      namedExports[key] = value;
    }
    return {
      default: fileModule.defaultExport,
      named: namedExports
    };
  };
}
function getMailerLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.mailer);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isMailer(m))
        throw new Error(
          `default export in ${modulePath} is not a valid mailer`
        );
      return m;
    });
    if (!fileModule)
      throw new Error(`no default export found in ${modulePath}`);
    if (!fileModule.defaultExport)
      throw new Error(`no default export found in ${modulePath}`);
    const namedExports = {};
    for (const [key, value] of Object.entries(fileModule.namedExports)) {
      if (!isMailer(value))
        throw new Error(
          `named export ${key} in ${modulePath} is not a valid mailer`
        );
      namedExports[key] = test() ? devMailer : value;
    }
    if (test()) {
      const log = getLogger("app-loader");
      log.warn("NODE_ENV=test, skipping mailer");
    }
    return {
      default: test() ? devMailer : fileModule.defaultExport,
      named: namedExports
    };
  };
}
function getJobsLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.jobs);
    const fileModules = await loadModulesFromDir(
      modulePath,
      async (m) => {
        if (!isJob(m))
          throw new Error(`default export in ${modulePath} is not a valid job`);
        return m;
      },
      [".ts"]
    );
    const jobs = {};
    for (const fileModule of fileModules) {
      if (!fileModule.defaultExport)
        throw new Error(`no default export found in ${modulePath}`);
      jobs[fileModule.filename] = fileModule.defaultExport;
    }
    return jobs;
  };
}
function getCommandsLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.commands);
    const fileModules = await loadModulesFromDir(
      modulePath,
      async (m) => {
        if (!isCommand(m))
          throw new Error(
            `default export in ${modulePath} is not a valid command`
          );
        return m;
      },
      [".ts"]
    );
    const commands = {};
    for (const fileModule of fileModules) {
      if (!fileModule.defaultExport)
        throw new Error(`no default export found in ${modulePath}`);
      commands[fileModule.filename] = fileModule.defaultExport;
    }
    return commands;
  };
}
function getSchedulesLoader(opts) {
  return async () => {
    const log = getLogger("app-loader");
    if (test()) {
      log.warn("NODE_ENV=test, skipping schedules");
      return {};
    }
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.schedules);
    const fileModules = await loadModulesFromDir(
      modulePath,
      async (m) => {
        if (!isSchedule(m))
          throw new Error(
            `default export in ${modulePath} is not a valid schedule`
          );
        return m;
      },
      [".ts"]
    );
    const schedules = {};
    for (const fileModule of fileModules) {
      if (!fileModule.defaultExport)
        throw new Error(`no default export found in ${modulePath}`);
      schedules[fileModule.filename] = fileModule.defaultExport;
    }
    return schedules;
  };
}
function getSeederLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.seed);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isSeeder(m))
        throw new Error(
          `default export in ${modulePath} is not a valid seeder`
        );
      return m;
    });
    if (!fileModule)
      throw new Error(`no default export found in ${modulePath}`);
    if (!fileModule.defaultExport)
      throw new Error(`no default export found in ${modulePath}`);
    return fileModule.defaultExport;
  };
}
function getRoutesLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.routes);
    const fileModules = await loadModulesFromDir(
      modulePath,
      async (m) => {
        if (!isHandler(m))
          throw new Error(
            `default export in ${modulePath} is not a valid handler`
          );
        return m;
      },
      [".tsx"]
    );
    return getFileRoutesFromFileModules(fileModules);
  };
}
function getLoggersLoader(opts) {
  return async () => {
    const { cwd: cwd3, config: config2 } = opts ?? {
      cwd: process.cwd(),
      config: await loadAndGetConfig()
    };
    const modulePath = join(cwd3, config2.paths.logger);
    const fileModule = await loadModule(modulePath, async (m) => {
      if (!isLogger(m)) throw new Error(`${modulePath} is not a valid logger`);
      return m;
    });
    if (!fileModule) throw new Error(`no module found in ${modulePath}`);
    const loggers = [];
    for (const [key, value] of Object.entries(fileModule.namedExports)) {
      if (!isLogger(value))
        throw new Error(
          `named export ${key} in ${modulePath} is not a valid logger`
        );
      loggers.push(value);
    }
    if (fileModule.defaultExport) loggers.push(fileModule.defaultExport);
    for (const logger of loggers) {
      consola3.addReporter(logger);
    }
    return loggers;
  };
}
function memoize(loader) {
  const log = getLogger("load-memoize");
  let result;
  return async () => {
    if (result) {
      log.debug("returning memoized app component", { loader: loader.name });
      return result;
    }
    log.debug("loading app component", { loader: loader.name });
    result = await loader();
    return result;
  };
}
function registerAppLoader(opts) {
  const log = getLogger("app-loader");
  log.debug("registering app loader", { opts });
  const database = memoize(getDatabaseLoader(opts));
  const server = memoize(getServerLoader(opts));
  const queue = memoize(getQueueLoader(opts));
  const mailer = memoize(getMailerLoader(opts));
  const jobs = memoize(getJobsLoader(opts));
  const commands = memoize(getCommandsLoader(opts));
  const schedules = memoize(getSchedulesLoader(opts));
  const seeder = memoize(getSeederLoader(opts));
  const routes = memoize(getRoutesLoader(opts));
  const loggers = memoize(getLoggersLoader(opts));
  setAppLoader({
    database,
    server,
    queue,
    mailer,
    jobs,
    commands,
    schedules,
    seeder,
    routes,
    loggers
  });
}

// src/bin/bin.ts
registerAppLoader();
void runCommand2({ cwd: cwd2() }).then(() => process.exit(0));
//# sourceMappingURL=bin.js.map